//
// Define default variables
//

currentVersion = "1.0.1";

route = "20";
direction = "West Bound";
stop = "4729";
predictionLimit = 4;
showBusNumbers = false;

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    dashcode.setupParts();
    getRoutes();
    versionCheck();
    showBusesCheck();
    setLimitOption();
    
    prePlural = getObj("prediction-pluraization");
    if (predictionLimit == 1) {
        prePlural.innerHTML = "Prediction"
    } else {
        prePlural.innerHTML = "Predictions"
    }
    
    timerInterval = null;
    if (timerInterval == null) {
        timerInterval = setInterval("getPredictions();", 40000);
    }
    
    if (firstLoad == true) {
        showBack();
        firstLoad = false;
    }
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
    widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("route"));
    widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("direction"));
    widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("stop"));
    widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("limit"));
    widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("showBusNumbers"));
    
    clearInterval(timerInterval);
    timerInterval = null;
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    clearInterval(timerInterval);
    
    if (timerInterval != null) {
        clearInterval(timerInterval);
        timerInterval = null;
    }
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    getPredictions();
    
    if (timerInterval == null) {
        timerInterval = setInterval("getPredictions();", 40000);
 	}
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{   

    versionCheck();
    
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        window.resizeTo (225, 220);	
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
    
    getRoutes();
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        window.resizeTo (225, calculateWidgetHeight());	
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}

//
// BUS TRACKER FUNCTIONS
//

function calculateWidgetHeight() {
	var height = 48 + // header
	             43;  // footer
	
	height += (predictionLimit * 35); // 35 row height
	
	return height;
}

if (window.widget) {
    if (widget.preferenceForKey(dashcode.createInstancePreferenceKey('route')) == undefined) {
        widget.setPreferenceForKey(route, dashcode.createInstancePreferenceKey('route'));
    }
    route = widget.preferenceForKey(dashcode.createInstancePreferenceKey('route'));
    
    if (widget.preferenceForKey(dashcode.createInstancePreferenceKey('direction')) == undefined) {
        widget.setPreferenceForKey(direction, dashcode.createInstancePreferenceKey('direction'));
    }
    direction = widget.preferenceForKey(dashcode.createInstancePreferenceKey('direction'));
    
    if (widget.preferenceForKey(dashcode.createInstancePreferenceKey('stop')) == undefined) {
        firstLoad = true;
        widget.setPreferenceForKey(stop, dashcode.createInstancePreferenceKey('stop'));
    }
    stop = widget.preferenceForKey(dashcode.createInstancePreferenceKey('stop'));
    
    if (widget.preferenceForKey(dashcode.createInstancePreferenceKey('limit')) == undefined) {
        widget.setPreferenceForKey(predictionLimit, dashcode.createInstancePreferenceKey('limit'));
    }
    predictionLimit = widget.preferenceForKey(dashcode.createInstancePreferenceKey('limit'));
    
    if (widget.preferenceForKey(dashcode.createInstancePreferenceKey('showBusNumbers')) == undefined) {
        widget.setPreferenceForKey(showBusNumbers, dashcode.createInstancePreferenceKey('showBusNumbers'));
    }
    showBusNumbers = widget.preferenceForKey(dashcode.createInstancePreferenceKey('showBusNumbers'));
}

function getObj(id) {
    return document.getElementById(id);
}

function setLimitOption() {
    getObj("limit").value = predictionLimit
}

function versionCheck() {
    compareVersion('http://widget.chicagobus.org/xml/version.xml');
}

function showBusesCheck() {
    showBusNumbersBox = getObj("showBusNumbersBox");
    if (showBusNumbers == true) {
        showBusNumbersBox.checked = true;
    }
}

function compareVersion(url) {
   xmlVersionReq = new XMLHttpRequest();
   xmlVersionReq.setRequestHeader("Cache-Control", "no-cache");
   xmlVersionReq.open("GET",url,true);
   xmlVersionReq.send(null);
   xmlVersionReq.onreadystatechange = function() {   
       if (null == xmlVersionReq.readyState) return;
       if (xmlVersionReq.readyState == 4) {
          if (xmlVersionReq.status == 200) {
                xml 				= xmlVersionReq.responseXML;
                version  			= xml.childNodes(0).firstChild.nodeValue;
                if (version <= currentVersion) {
                  msgObj			= getObj("version");
                  msgObj.innerHTML	= 'Version ' + currentVersion;
                } else { 
                  msgObj			= getObj("version");
                  msgObj.innerHTML	= '<a href="#" onclick="openWidgetSite()">Download New Version</a>'; 
                }
          } else {
             if (null == xmlVersionReq.status) return;
          }
       }
    }
}

function openWidgetSite(url)
{
	widget.openURL("http://widget.chicagobus.org");
}

function changeRoute(elem) {
    route = elem.value;
    if (window.widget) widget.setPreferenceForKey(route, dashcode.createInstancePreferenceKey('route'));
    direction = "";
    stop = "";
    getDirections();
}

function changeDirection(elem) {
    direction = elem.value;
    if (window.widget) widget.setPreferenceForKey(direction, dashcode.createInstancePreferenceKey('direction'));
    stop = "";
    getStops();
}

function changeStop(elem) {
    stop = elem.value;
    if (window.widget) widget.setPreferenceForKey(stop, dashcode.createInstancePreferenceKey('stop'));
    getPredictions();
}

function changeLimit(elem) {
    limit = elem.value;
    if (window.widget) widget.setPreferenceForKey(limit, dashcode.createInstancePreferenceKey('limit'));
    predictionLimit = widget.preferenceForKey(dashcode.createInstancePreferenceKey('limit'));
    
    prePlural = getObj("prediction-pluraization");
    if (predictionLimit == 1) {
        prePlural.innerHTML = "Prediction"
    } else {
        prePlural.innerHTML = "Predictions"
    }
    
    getPredictions();
}

function changeShowBusNumbers(elem) {
    if (elem.checked == true) {
        showBusNumbers = true;
    } else {
        showBusNumbers = false;
    }

    if (window.widget) widget.setPreferenceForKey(showBusNumbers, dashcode.createInstancePreferenceKey('showBusNumbers'));
    showBusNumbers = widget.preferenceForKey(dashcode.createInstancePreferenceKey('showBusNumbers'));
    getPredictions();
}

function getRoutes() {
    loadXML('http://chicago.transitapi.com/bustime/eta/routeDirectionStopAsXML.jsp', 'routes');
}

function getDirections() {
    loadXML('http://chicago.transitapi.com/bustime/eta/routeDirectionStopAsXML.jsp?route=' + route + '', 'directions');
}

function getStops() {
    loadXML('http://chicago.transitapi.com/bustime/eta/routeDirectionStopAsXML.jsp?route=' + route + '&direction=' + direction + '', 'stops');
}

function getPredictions() {
    loadXML('http://ctabustracker.com/bustime/map/getStopPredictions.jsp?stop=' + stop + '&route=' + route +'', 'predictions');
}

function updateTimeStamp() {
    var curDate = new Date();
   
    msgObj   	   		= getObj("msg");
    msgObj.innerText	= ("As of %@").replace("%@", curDate.toLocaleTimeString("short"));
}

function loadXML(url,type) {
   xmlRequest = new XMLHttpRequest();
   xmlRequest.setRequestHeader("Cache-Control", "no-cache");
   xmlRequest.open("GET",url,true);
   xmlRequest.setRequestHeader("Content-Type", "text/xml");
   xmlRequest.overrideMimeType("text/xml");
   xmlRequest.send(null);
   xmlRequest.onreadystatechange = function() {   
       if (null == xmlRequest.readyState) return;
       if (xmlRequest.readyState == 4) {
          if (xmlRequest.status == 200) {      
             parseXML(type);
          } else {
             if (null == xmlRequest.status) return;
          }
       }
    }
}

function parseXML(type) {
   if (null == xmlRequest.responseXML) {
      msgObj			= getObj("msg");
      msgObj.innerHTML	= "Error loading data";
   } else {
      xml = xmlRequest.responseXML;
         switch (type) {
            case 'routes':
              routes			    = xml.getElementsByTagName("route");
              routesHTML		    = "";
              for (var i = 0; i < routes.length; i++) {
                 routesHTML	       += parseRoute(routes[i]);
              }
              routes   	            = getObj("routes");
              routes.innerHTML      = routesHTML;
              if (route != "") { getObj("routes").value = route } else { route = getObj("routes").value }
              if (window.widget) widget.setPreferenceForKey(route, dashcode.createInstancePreferenceKey('route'));
              getDirections();
              break;
            case 'directions':
              directions 	 		= xml.getElementsByTagName("direction");
              directionsHTML		= "";
              for (var i = 0; i < directions.length; i++) {
                 directionsHTML	   += parseDirections(directions[i]);
              }
              directions   	        = getObj("directions");
              directions.innerHTML  = directionsHTML;
              if (direction != "") { getObj("directions").value = direction } else { direction = getObj("directions").value }
              if (window.widget) widget.setPreferenceForKey(direction, dashcode.createInstancePreferenceKey('direction'));
              getStops();
              break;
            case 'stops':
              stops 	 			= xml.getElementsByTagName("stop");
              stopsHTML				= "";
              for (var i = 0; i < stops.length; i++) {
                 stopsHTML	       += parseStops(stops[i]);
              }
              stops   	      		= getObj("stops");
              stops.innerHTML		= stopsHTML;
              if (stop != "") { getObj("stops").value = stop } else { stop = getObj("stops").value }
              if (window.widget) widget.setPreferenceForKey(stop, dashcode.createInstancePreferenceKey('stop'));
              getPredictions();
              break;
            case 'predictions':
                  // fetch the stop name
                  stopHTML  			    = xml.childNodes(0).childNodes(3).firstChild.nodeValue;
                  stopObj   	   	  	    = getObj("stop");
                  stopObj.innerHTML			= stopHTML;
                  // display notice if no predictions
              if (xml.childNodes(0).childNodes(7).nodeName == "noPredictionMessage") {
                  routeObj					= getObj("route");
                  routeObj.innerHTML	   	= route;
                  directionObj				= getObj("direction");
                  directionObj.innerHTML	= direction;
                  predictions.innerHTML	 	= '<li class=\"error\">' + xml.childNodes(0).childNodes(7).firstChild.nodeValue + '</li>';
                  remaining = predictionLimit - 1;
                  for (var i = 0; i < remaining; i++) {
                    predictions.innerHTML  += "<li></li>";
                  }
              } else {
                  // fetch the route number
                  routeHTML					= xml.childNodes(0).childNodes(5).childNodes(1).firstChild.nodeValue;
                  routeObj					= getObj("route");
                  routeObj.innerHTML	   	= routeHTML;
                  // fetch the direction
                  directionHTML				= xml.childNodes(0).childNodes(5).childNodes(3).firstChild.nodeValue;
                  directionObj				= getObj("direction");
                  directionObj.innerHTML	= directionHTML;
                  // fetch the prediction times
                  predictions 	 			= xml.getElementsByTagName("pre");
                  predictionsHTML			= "";
                  // determine how many predictions to display
                  if (predictionLimit) {
                        if (predictions.length < predictionLimit) {
                        limit = predictions.length;
                    } else { limit = predictionLimit; }
                  } else { limit = predictions.length; }
                  // parse the predictions
                  for (var i = 0; i < limit; i++) {
                     predictionsHTML	   += parsePredictions(predictions[i]);
                  } 
                  // display the predictions
                  predictions   		    = getObj("predictions");
                  if (predictionsHTML == "") {
                    predictions.innerHTML	= "<li class=\"error\">No arrival times available.</li>";
                    remaining = predictionLimit - 1;
                    for (var i = 0; i < remaining; i++) {
                      predictions.innerHTML  += "<li></li>";
                    }
                  } else {
                    if (limit < predictionLimit) {
                     remaining = predictionLimit - limit;
                     for (var i = 0; i < remaining; i++) {
                      predictionsHTML       += "<li></li>";
                     }
                    }
                    predictions.innerHTML	= predictionsHTML;
                  }
              }
              updateTimeStamp();
            break;
        }
   }
}		

function parseRoute(routes) {
    var name		= routes.attributes.getNamedItem("name").nodeValue
    var number		= routes.attributes.getNamedItem("designator").nodeValue
    
    return '<option value=\"' + number + '\">' + number + ' ' + name + '</option>';
} 

function parseDirections(directions) {
    var direction		= directions.attributes.getNamedItem("name").nodeValue
    
    return '<option value=\"' + direction + '\">' + direction + '</option>';
} 

function parseStops(stops) {
    var stop		= stops.attributes.getNamedItem("name").nodeValue
    var stopid		= stops.attributes.getNamedItem("id").nodeValue
    
    return '<option value=\"' + stopid + '\">' + stop + '</option>';
} 

function parsePredictions(prediction) {
    var predictionTime		= prediction.childNodes[1].firstChild.nodeValue;
    var finalDestination	= prediction.childNodes[3].firstChild.nodeValue;
    var vehicle				= prediction.childNodes[5].firstChild.nodeValue;
    
    if (predictionTime == "APPROACHING") {
        var prediction = "<span class=\"predictionValue now\">Now</span>";
    } else {
        predictionValue = predictionTime.replace(/[^0-9]/g, '');
        predictionUnit 	= "Min";
        prediction 		= '<span class="predictionValue">' + predictionValue + '</span><span class="predictionUnit">' + predictionUnit + '</span>';				
    }
    
    if (showBusNumbers == true) {
    return '<li><div id=\"prediction\">' + prediction + '</div><div id=\"destination-showbuses\">To <span class="destinationName">' + finalDestination + '</span></div><div id="vehicle">#' + vehicle + '</div></li>';
    } else {
        checkWhiteSpace = new RegExp(/\s/);
        if (checkWhiteSpace.test(finalDestination)) {
        return '<li><div id=\"prediction\">' + prediction + '</div><div id=\"destination\">To <span class="destinationName">' + finalDestination + '</span></div></li>';
        } else {
        if (finalDestination.length > 14) {
            splitDestination = finalDestination.replace(/\//g, '/<br/>');
        } else { splitDestination = finalDestination }
        return '<li><div id=\"prediction\">' + prediction + '</div><div id=\"destination\">To <span class="destinationName">' + splitDestination + '</span></div></li>'; }
    }
}